# Modified stationary m-state CMPHMM (used for Fuerth data (m=2) and for Guttorp data (m=4)).

library(COMPoissonReg)
logit=function(p) log(p/(1-p))
logitinv=function(x) exp(x)/(1+exp(x))

### Transforming natural parameters to working parameters
mod.pn2w = function(p,lambda,nu,gamma){
  tp      = logit(p)
  tlambda = log(lambda)
  tnu     = log(nu)
  foo     = log(gamma/diag(gamma))
  tgamma  = as.vector(foo[!diag(m)])
  parvect = c(tp,tlambda,tnu,tgamma)
  return(parvect) }

### Transforming working parameters to natural parameters 
mod.pw2n = function(parvect){
  p             = logitinv(parvect[1])
  lambda        = exp(parvect[2:m])
  nu            = exp(parvect[(m+1):(2*m-1)])
  gamma         = diag(m)
  gamma[!gamma] = exp(parvect[(2*m):(m*m+m-1)])
  gamma         = gamma/apply(gamma,1,sum)
  delta         = solve(t(diag(m)-gamma+1),rep(1,m))
  return(list(p=p,lambda=lambda,nu=nu,gamma=gamma,delta=delta)) }

### Computing minus the log-likelihood from the working parameters
mod.mllk = function(parvect,x,m,...){
  n       = length(x)
  np      = mod.pw2n(parvect)
  p       = np$p
  lambda  = np$lambda
  nu      = np$nu
  gamma   = np$gamma
  delta   = np$delta
  foo     = np$delta
  lscale  = 0
  for (i in 1:n){  
    if(x[i]>1) {temp=0}
    else       {temp=p^{x[i]}*(1-p)^{1-x[i]}}
    P      = c (temp, dcmp(x[i],lambda,nu) )
    foo    = foo %*% gamma*P
    sumfoo = sum(foo)
    lscale = lscale+log(sumfoo)
    foo    = foo/sumfoo }
  -lscale }

#######################################Fuerth pedestrian data#################################
xx=scan("Fuerth_pedestrians_505.txt")

m=2
(startn=c(0.1914,0.1070,0.8862,9.1647,2.400))
# Starting values come from the 2-state (un-modified) CMPHMM.

(p0=startn[3]/(1+startn[3]))
(lambda0=startn[4])
(nu0=startn[5])
string=c(1-startn[1],startn[1],startn[2],1-startn[2])
(gamma0=matrix(string,nro=m,byro=T))

(startw=mod.pn2w(p0,lambda0,nu0,gamma0))
(nop=length(startw))  
(modmod=nlm(mod.mllk,startw,x=xx,print.level=2,iterlim=1000)) 
mod.pw2n(modmod$estimate)
(mllk=modmod$minimum)
(AIC=2*(mllk+nop))
(n=length(xx))
(BIC=2*mllk+nop*log(n))

#######################################################################
# Calculating the standard errors of the estimated parameters:

### Computing minus the log-likelihood from the natural parameters
mod.mllk_np = function(parvect,x,m,...){
  n             = length(x)
  p             = parvect[1]
  lambda        = parvect[2:m]
  nu            = parvect[(m+1):(2*m-1)]
  gamma         = diag(m)
  gamma[!gamma] = parvect[(2*m):(m*m+m-1)]
  gamma         = t(gamma)
  for (i in 1:m) gamma[i,i] = 1-sum(gamma[i,-i])
  delta         = solve(t(diag(m)-gamma+1),rep(1,m))
  foo           = delta
  lscale        = 0
  for (i in 1:n){  
    if(x[i]>1) {temp=0}
    else {temp=p^{x[i]}*(1-p)^{1-x[i]}}
    P       = c(temp,dcmp(x[i],lambda,nu))
    foo     = foo %*% gamma*P
    sumfoo  = sum(foo)
    lscale  = lscale+log(sumfoo)
    foo     = foo/sumfoo }
  return(-lscale) }

parvect0=c(0.4698,9.1647,2.400,0.1914,0.1070)
(mod.mod_np=nlm(mod.mllk_np,parvect0,xx,m=2,print.level=2,iterlim=1000,hessian=T)) 
round(mod.mod_np$estimate,4)
std_errors=sqrt(diag(solve(mod.mod_np$hessian)))
round(std_errors,4)

######################################Guttorp gold data#######################################
xx=scan("Guttorp_gold_1598.txt") # Guttorp v. of gold particles.  (1598)

m=4
lambda0=c(0.0403, 3.05, 119, 224)
(p0=lambda0[1]/(1+lambda0[1]))
(lambda0=lambda0[-1])
(gamma0=matrix(c(0.737, 0.216, 0.046, 0.001,
                  0.079, 0.803, 0.114, 0.004,
                  0.001, 0.194, 0.733, 0.072, 
                  0.029, 0.001, 0.165, 0.805),
               m,m,byrow=TRUE))
(nu0=c(4.22, 5.29, 3.89))
# Above starting values come from the 4-state (unmodified) CMPHMM.

(startw=mod.pn2w(p0,lambda0,nu0,gamma0))
(modmod=nlm(mod.mllk,startw,x=xx,print.level=2,iterlim=1000))
mod.pw2n(modmod$estimate)
(mllk=modmod$minimum)
(nop=length(startw))  
(AIC=2*(mllk+nop))
(n=length(xx))
(BIC=2*mllk+nop*log(n))